
local _,L = ...
local rematch = Rematch
local settings
local saved

function rematch:InitSaveAs()
	settings = RematchSettings
	saved = RematchSaved
	RegisterAddonMessagePrefix("Rematch")
	rematch:RegisterEvent("CHAT_MSG_ADDON")
end

-- this displays a dialog to save the sidelined team
-- note: a team must be sidelined before saving it!
function rematch:ShowSaveDialog()

	local dialog = rematch.dialog

	rematch:ShowDialog("SaveTeam",152,L["Save As..."],L["Save this team?"],rematch.SaveDialogAcceptOnClick)

	local team, teamKey = rematch:GetSideline()
	rematch:FillPetFramesFromTeam(dialog.team.pets,team)

	dialog.team:SetPoint("TOP",0,-48)
	dialog.team:Show()

	dialog.editBox:SetPoint("TOP",0,-20)
	dialog.editBox:SetWidth(200)
	dialog.editBox:SetText(rematch:GetSidelineTitle())
	dialog.editBox:SetScript("OnTextChanged",rematch.SaveEditBoxOnTextChanged)
	dialog.editBox:Show()

	-- tabPicker button appears if user has custom tabs defined--to choose where to save team
	if #settings.TeamGroups>1 then
		rematch:SelectTeamTab(team.tab or 1)
		dialog.tabPicker:Show()
	end

	-- if any leveling pets, setup preferencesPanel
	if team[1][1]==0 or team[2][1]==0 or team[3][1]==0 then
		dialog.hasLeveling = true
		dialog.preferencesPanel:Show()
		rematch:UpdatePreferences()
		if team.minHP or team.maxXP then
			dialog.showPreferences = true -- if any preferences defined, show panel from start
		end
		-- make tab on reused editBox jump to minHP (this is cleared when dialogs are repurposed)
		dialog.editBox:SetScript("OnTabPressed",function(self)
			if dialog.preferencesPanel:IsVisible() then
				dialog.preferencesPanel.minHP:SetFocus(true)
			end
		end)
		-- display toggle button in lower left of dialog
		dialog.preferencesToggle:SetPoint("BOTTOMLEFT",3,2)
		dialog.preferencesToggle:Show()
		-- if saving from current pets, show the option 'Save leveling pets as themselves'
		if not dialog.asThemselves then
			dialog.asThemselves = CreateFrame("CheckButton",nil,dialog,"RematchCheckButtonTemplate")
			rematch:RegisterDialogWidget("asThemselves")
			dialog.asThemselves.text:SetText(L["Save leveling pets as themselves"])
			dialog.asThemselves.tooltipTitle = L["Save As Themselves"]
			dialog.asThemselves.tooltipBody = L["Save pets without turning them into leveling pets.\n\nSo loading this team in the future will load these specific pets and not from the queue."]
			dialog.asThemselves:SetScript("OnClick",rematch.PreferencesAsThemselvesOnClick)
		end
		dialog.asThemselves:SetChecked(false) -- always default it to off
		dialog.asThemselves:SetPoint("BOTTOMLEFT",24,40)
	else
		dialog.hasLeveling = nil
	end

	-- if saving with an npcID attached, add a checkbox to allow clearing the npcID
	if type(teamKey)=="number" then
		dialog.checkBox.text:SetText(L["Save NPC ID with team"])
		dialog.checkBox:SetChecked(true)
		dialog.checkBox.tooltipTitle = format(L["Save NPC ID (%d)"],teamKey)
		dialog.checkBox.tooltipBody = L["Save the NPC's unique identifier for this team so Rematch will react only to this version of the NPC.\n\nIf an ID is not saved, Rematch will react to any targets that share this team's name.\n\nTeams with an NPC ID are listed in \124cffffffffWhite\124r and can share names with other teams.\nTeams without an NPC ID are listed in \124cffffd200Gold\124r and must be uniquely named."]
		dialog.checkBox:SetPoint("TOPLEFT",dialog.team,"BOTTOMLEFT",35,-6)
		dialog.keepNpcID = true
		dialog.checkBox:SetScript("OnClick",rematch.SaveNpcIDOnClick)
		dialog.checkBox:Show()
	end

	RematchTeamCard:Hide()
end

function rematch:UpdateSaveDialog()
	local team, teamKey = rematch:GetSideline()
	local originalKey = rematch:GetSidelineContext("originalKey")
	local dialog = rematch.dialog
	local warnOffset

	-- if team has a named key, that's different than the original key, and it's the same as
	-- an existing team, display a warning that a team already has this name
	-- (for import and receive, originalKey should be nil)
	if type(teamKey)=="string" and teamKey~=originalKey and saved[teamKey] then
		dialog.warning.text:SetText(L["A team already has this name."])
		dialog.warning:SetPoint("TOP",0,-42)
		dialog.warning:Show()
		warnOffset = 18
	else
		dialog.warning:Hide()
		warnOffset = 0
	end

	if type(teamKey)=="number" then
		dialog.editBox:SetTextColor(1,1,1)
	else
		dialog.editBox:SetTextColor(1,.82,0)
	end

	local npcOffset = (type(teamKey)=="number" or type(originalKey)=="number") and 26 or 0
	local showPreferences = dialog.showPreferences and dialog.hasLeveling
	local showAsThemselves = showPreferences and rematch:GetSidelineContext("forCurrent")
	dialog.preferencesPanel:SetShown(showPreferences and true)
	if dialog.asThemselves then
		dialog.asThemselves:SetShown(showAsThemselves and true)
	end
	dialog:SetHeight(152 + npcOffset + warnOffset + (showPreferences and (dialog.preferencesPanel:GetHeight()+(showAsThemselves and 24 or 0)) or 0))
	dialog.team:SetPoint("TOP",0,-48-warnOffset)
	dialog.preferencesPanel:SetPoint("TOP",0,-120-warnOffset-npcOffset)
end

function rematch:SaveEditBoxOnTextChanged()
	local teamName = self:GetText()
	rematch:SetAcceptEnabled(teamName:len()>0)
	local team,teamKey = rematch:GetSideline()
	if type(teamKey)=="number" then -- teamKey is an npcID, name can be anything (duplicate copies fine too)
		team.teamName = teamName
	else -- teamKey is named, change name of key in sideline team
		rematch:ChangeSidelineKey(teamName)
	end
	rematch:UpdateSaveDialog()
end

-- this is the checkbutton click for "Save NPC ID with team"
function rematch:SaveNpcIDOnClick()
	rematch:HideTooltip()
	local team,teamKey = rematch:GetSideline()
	local originalKey = rematch:GetSidelineContext("originalKey")
	local isChecked = self:GetChecked()
	if isChecked then -- save npcID: change the key back to its original key
		rematch:ChangeSidelineKey(originalKey)
	else -- do not save npcID: rename team to current editbox
		rematch:ChangeSidelineKey(rematch.dialog.editBox:GetText())
	end
	rematch:UpdateSaveDialog()
end

-- when accept (green check) clicked on main save dialog
function rematch:SaveDialogAcceptOnClick()
	local team,teamKey = rematch:GetSideline()
	local askReplace
	if saved[teamKey] then
		for i=1,3 do
			if saved[teamKey][i][1]~=team[i][1] then
				askReplace = true
			end
		end
	end
	if not askReplace then -- okay to push as is, save it!
		rematch:PushSideline(true)
	else -- ask to confirm whether to save this team
		rematch:ShowReplaceDialog(rematch.CancelReplaceSaveDialog)
	end
end

-- when replace dialog is summoned from main save dialog, a cancel
-- of the replace dialog will run this, bringing up the save dialog again
function rematch:CancelReplaceSaveDialog()
	local dialog = rematch.dialog
	if dialog.asThemselves and dialog.asThemselves:GetChecked() then
		dialog.asThemselves:Click()
	end
	rematch:ShowSaveDialog()
end

-- shows the replace dialog; accepting will push the sideline team
-- cancel will run the passed cancelFunc (typically to return the previous dialog)
function rematch:ShowReplaceDialog(cancelFunc)
	local team,teamKey = rematch:GetSideline()
	local dialog = rematch:ShowDialog("ReplaceTeam",220,rematch:GetTeamTitle(teamKey,true),L["Overwrite this team?"],function() rematch:PushSideline(true) end)
	if not dialog.replace then
		dialog.replace = CreateFrame("Frame",nil,dialog,"RematchTeamTemplate")
		rematch:RegisterDialogWidget("replace")
		dialog.replace.arrow = dialog.replace:CreateTexture(nil,"ARTWORK")
		dialog.replace.arrow:SetSize(32,32)
		dialog.replace.arrow:SetPoint("TOP",dialog.replace,"BOTTOM",0,0)
		dialog.replace.arrow:SetTexture("Interface\\Buttons\\UI-MicroStream-Green")
	end
	-- old team is saved[teamKey] and filled into dialog.replace
	dialog.replace:SetPoint("TOP",0,-24)
	rematch:FillPetFramesFromTeam(dialog.replace.pets,saved[teamKey])
	dialog.replace:Show()

	-- new team is sideline team and filled into dialog.team
	dialog.team:SetPoint("TOP",dialog.replace,"BOTTOM",0,-32)
	rematch:FillPetFramesFromTeam(dialog.team.pets,team)
	dialog.team:Show()
	dialog.runOnCancel = cancelFunc
end

--[[ Rename ]]

-- called from rmf
function rematch:ShowRenameDialog(key)
	local team,teamKey = rematch:SetSideline(key)
	local dialog = rematch:ShowDialog("RenameTeam",154,rematch:GetTeamTitle(teamKey,true),L["Rename this team?"],rematch.RenameAccept)
	dialog.team:SetPoint("TOP",0,-24)
	rematch:FillPetFramesFromTeam(dialog.team.pets,team)
	dialog.team:Show()

	dialog.editBox:SetPoint("TOP",dialog.team,"BOTTOM",0,-8)
	if type(teamKey)=="number" then
		dialog.editBox:SetTextColor(1,1,1)
	else
		dialog.editBox:SetTextColor(1,.82,0)
	end
	dialog.editBox:SetWidth(200)
	dialog.editBox:SetText(rematch:GetTeamTitle(teamKey))
	dialog.editBox:SetScript("OnTextChanged",rematch.RenameOnTextChanged)
	dialog.editBox:Show()
end

function rematch:RenameOnTextChanged()
	local dialog = rematch.dialog
	local teamName = self:GetText()
	local team,teamKey = rematch:GetSideline()
	local nameTaken = type(teamKey)~="number" and saved[teamName] and teamName~=rematch:GetSidelineContext("originalKey")
	rematch:SetAcceptEnabled(teamName:len()>0 and not nameTaken)
	if nameTaken then
		dialog.warning:SetPoint("TOP",dialog.editBox,"BOTTOM",0,-4)
		dialog.warning.text:SetText(L["A team already has this name."])
		dialog.warning:Show()
		dialog:SetHeight(178)
	else
		dialog.warning:Hide()
		dialog:SetHeight(154)
	end
	if type(teamKey)=="number" then -- teamKey is an npcID, name can be anything (duplicate copies fine too)
		team.teamName = teamName
	else -- teamKey is named, change name of key in sideline team
		rematch:ChangeSidelineKey(teamName)
	end
end

function rematch:RenameAccept()
	local team,teamKey = rematch:GetSideline()
	local originalKey = rematch:GetSidelineContext("originalKey")
	saved[originalKey] = nil
	rematch:PushSideline()
	rematch:UpdateWindow()
	rematch:ScrollToTeam(teamKey)
end

--[[ Sharing teams: Export/Import & Send/Receive ]]

-- common functions between sharing methods

-- Name of team:npcID:S1:A1:A2:A3:S2:A1:A2:A3:S3:A1:A2:A3:
function rematch:ConvertTeamToString()
	local team,teamKey = rematch:GetSideline()
	local teamString
	if type(teamKey)=="number" then
		teamString = format("%s:%s:",team.teamName,teamKey)
	else
		teamString = format("%s:0:",teamKey)
	end
	for i=1,3 do
		local petID = team[i][1]
		if not petID then
			petID = 1 -- empty slot is "speciesID" 1
		elseif type(petID)=="number" then
			petID = petID -- already a speciesID, or 0 for leveling slot
		elseif team[i][5] then
			petID = team[i][5] -- a genuine petID should have speciesID in [5]
		else
			petID = 1
		end
		teamString = teamString..format("%d:%d:%d:%d:",petID,team[i][2],team[i][3],team[i][4])
	end
	return teamString
end

-- sidelines a team defined in teamString, returning the key if it was valid
function rematch:ConvertStringToTeam(teamString)
	local t={{},{},{}}
	local teamName
	teamName,t[4],t[1][1],t[1][2],t[1][3],t[1][4],t[2][1],t[2][2],t[2][3],t[2][4],t[3][1],t[3][2],t[3][3],t[3][4] = teamString:match("(.+):(%d+):(%d+):(%d+):(%d+):(%d+):(%d+):(%d+):(%d+):(%d+):(%d+):(%d+):(%d+):(%d+):")
	if teamName then
		for i=1,3 do
			-- convert all string matches to numbers
			for j=1,4 do
				t[i][j]=tonumber(t[i][j])
			end
			-- copy speciesID of actual species to [5]
			if t[i][1]>1 then
				t[i][5] = t[i][1]
			end
			-- convert petID's of 1 to 0x0000etc (emptyPetID)
			if t[i][1]==1 then
				t[i][1] = rematch.emptyPetID
			end
		end
		rematch:ValidateTeam(t) -- fill out species IDs with petIDs
		local key = teamName
		if t[4]~="0" then
			key = tonumber(t[4])
			t.teamName = teamName
		end
		t.tab = RematchSettings.SelectedTab
		rematch:SetSideline(key,t)
		return key
	end
end

-- For import and receive windows, this creates a little frame with a warning
-- and two radio options: (*) Create a new copy. ( ) Overwrite existing team.
-- Sideline context "overwrite" is true when second radio button selected.
-- This should NOT be used as a dialog widget since the receive window needs
-- to be visible when Rematch is not.
-- returns true if the confirmation window is displayed
function rematch:ConfirmKeyExists(key,anchorPoint,relativeTo,relativePoint,xoff,yoff)
	if not key or not saved[key] then
		if rematch.overwriteConfirm then
			rematch.overwriteConfirm:Hide()
		end
		return false
	end
	-- if we're at this point a team exists with the key, display radio buttons to ask what to do with it
	local confirm = rematch.overwriteConfirm
	if not confirm then -- create overwriteConfirm if it doesn't exist
		rematch.overwriteConfirm = CreateFrame("Frame")
		confirm = rematch.overwriteConfirm
		confirm:SetSize(200,66)
		confirm.text = confirm:CreateFontString(nil,"ARTWORK","GameFontHighlight")
		confirm.text:SetPoint("TOP",16,-4)
		local icon = confirm:CreateTexture(nil,"ARTWORK")
		icon:SetSize(22,22)
		icon:SetPoint("RIGHT",confirm.text,"LEFT",-6,0)
		icon:SetTexture("Interface\\DialogFrame\\UI-Dialog-Icon-AlertNew")
		confirm.copy = CreateFrame("CheckButton",nil,confirm,"UIRadioButtonTemplate")
		confirm.copy.text:SetFontObject("GameFontHighlight")
		confirm.copy.text:SetText(L["Save as a new team."])
		confirm.copy:SetPoint("TOPLEFT",12,-26)
		confirm.copy:SetHitRectInsets(0,-160,0,0)
		confirm.copy:SetChecked(true)
		confirm.copy:SetScript("OnClick",function(self) rematch:SetSidelineContext("overwrite",nil) rematch:UpdateConfirmRadios(self) end)
		confirm.overwrite = CreateFrame("CheckButton",nil,confirm,"UIRadioButtonTemplate")
		confirm.overwrite.text:SetFontObject("GameFontHighlight")
		confirm.overwrite.text:SetText(L["Overwrite existing team."])
		confirm.overwrite:SetPoint("TOPLEFT",confirm.copy,"BOTTOMLEFT",0,-2)
		confirm.overwrite:SetHitRectInsets(0,-160,0,0)
		confirm.overwrite:SetScript("OnClick",function(self) rematch:SetSidelineContext("overwrite",true) rematch:UpdateConfirmRadios(self) end)
		confirm:SetScript("OnHide",function(self) self:Hide() end) -- really hide this frame if any parent hides
	end
	if type(key)=="number" then
		confirm.text:SetText(L["This target already has a team."])
	else
		confirm.text:SetText(L["A team already has this name."])
	end
	confirm:ClearAllPoints()
	confirm:SetPoint(anchorPoint,relativeTo,relativePoint,xoff,yoff)
	confirm:SetParent(relativeTo)
	rematch:UpdateConfirmRadios()
	confirm:Show()
	return true
end

-- updates the two radio buttons to make sure only one is checked
-- radio is the passed button itself
function rematch:UpdateConfirmRadios(radio)
	local confirm = rematch.overwriteConfirm
	if not radio then -- if radio isn't passed, it's the radio that's presently checked
		radio = confirm.copy:GetChecked() and confirm.copy or confirm.overwrite
	end
	if radio==confirm.copy then
		confirm.copy:SetChecked(true)
		confirm.overwrite:SetChecked(false)
		confirm.copy.text:SetTextColor(1,1,1)
		confirm.overwrite.text:SetTextColor(.6,.6,.6)
	else
		confirm.copy:SetChecked(false)
		confirm.overwrite:SetChecked(true)
		confirm.copy.text:SetTextColor(.6,.6,.6)
		confirm.overwrite.text:SetTextColor(1,1,1)
	end
end

--[[ Export ]]

-- called from rmf
function rematch:ShowExportDialog(key)
	local team,teamKey = rematch:SetSideline(key)
	local dialog = rematch:ShowDialog("ExportTeam",244,rematch:GetSidelineTitle(true),"",true)
	dialog.team:SetPoint("TOP",0,-24)
	dialog.team:Show()
	rematch:FillPetFramesFromTeam(dialog.team.pets,team)
	dialog.text:SetSize(180,40)
	dialog.text:SetPoint("TOP",dialog.team,"BOTTOM",0,-8)
	dialog.text:SetText(L["Press CTRL+C to copy this team to the clipboard."])
	dialog.text:Show()
	dialog.multiLine:SetPoint("TOP",dialog.text,"BOTTOM",0,-8)
	dialog.multiLine:Show()
	dialog.multiLine.editBox:SetText(rematch:ConvertTeamToString())
	dialog.multiLine.editBox:HighlightText(0)
	dialog.multiLine.editBox:SetScript("OnTextChanged",rematch.ExportOnTextChanged)
end
-- any changes to editbox reverts editbox to sidelined team as a string
function rematch:ExportOnTextChanged()
	self:SetText(rematch:ConvertTeamToString())
	self:HighlightText(0)
end

--[[ Import ]]

function rematch:ShowImportDialog()
	local dialog = rematch.dialog
	if dialog.name == "ImportTeam" then
		rematch:HideDialogs()
		return
	end
	rematch:ShowDialog("ImportTeam",244,L["Import Team"],"",rematch.ImportAcceptOnClick)
	dialog.team:SetPoint("TOP",0,-24)
	rematch:WipePetFrames(dialog.team.pets)
	dialog.team:Show()
	dialog.text:SetSize(180,40)
	dialog.text:SetPoint("TOP",dialog.team,"BOTTOM",0,-8)
	dialog.text:Show()
	dialog.multiLine:SetPoint("BOTTOM",0,48)
	dialog.multiLine:Show()
	dialog.multiLine.editBox:SetScript("OnTextChanged",rematch.ImportOnTextChanged)
	if #settings.TeamGroups>1 then
		dialog.tabPicker:Show()
	end
	rematch:SetAcceptEnabled(false) -- initially accept button disabled
end

function rematch:ImportOnTextChanged()
	local dialog = rematch.dialog
	local key = rematch:ConvertStringToTeam(self:GetText())
	rematch:SetAcceptEnabled(key and true)
	local confirm = rematch:ConfirmKeyExists(key,"TOP",dialog.team,"BOTTOM",0,-8)
	dialog:SetHeight(confirm and 268 or 244)
	dialog.text:SetText(confirm and "" or L["Press CTRL+V to paste a team from the clipboard."])
	if self:GetText():len()>0 then
		dialog.teamString = self:GetText()
	end
	if key then
		local team,teamKey = rematch:GetSideline()
		rematch:FillPetFramesFromTeam(dialog.team.pets,team)
		dialog.header.text:SetText(rematch:GetSidelineTitle(true))
		dialog.prompt:SetText(L["Save this team?"])
	else
		dialog.header.text:SetText(L["Import As..."])
		rematch:WipePetFrames(dialog.team.pets)
		dialog.prompt:SetText("")
	end
end

-- when accept (green check) is clicked on the import dialog
function rematch.ImportAcceptOnClick()
	if rematch.overwriteConfirm and rematch.overwriteConfirm.copy:GetChecked() then
		rematch:MakeSidelineUnique()
	end
	local team,teamKey = rematch:GetSideline()
	if saved[teamKey] then
		rematch:ShowReplaceDialog(rematch.CancelReplaceImportDialog)
	else
		rematch:PushSideline(true)
	end
end

-- when the replace dialog is cancelled while overwriting an existing team
-- from an import; restores the import dialog from where it left off
function rematch:CancelReplaceImportDialog()
	rematch:ShowImportDialog()
	rematch.dialog.multiLine.editBox:SetText(rematch.dialog.teamString)
end

--[[ Send ]]

-- called from rmf where it should've set sideline; warning passed to show a
-- warning message, such as "They're busy. Try again later."
function rematch:ShowSendDialog(warning)
	local dialog = rematch:ShowDialog("SendTeamStart",200,rematch:GetSidelineTitle(true),L["Send this team?"],rematch.SendTeam)
	dialog.team:SetPoint("TOP",0,-24)
	dialog.team:Show()
	dialog.text:SetSize(180,40)
	dialog.text:SetPoint("TOP",dialog.team,"BOTTOM",0,-8)
	dialog.text:SetText(L["Who would you like to send this team to?"])
	dialog.text:Show()
	dialog.editBox:SetPoint("TOP",dialog.text,"BOTTOM",0,-4)
	dialog.editBox:Show()
	rematch:FillPetFramesFromTeam(dialog.team.pets,(rematch:GetSideline()))
	rematch:SetAcceptEnabled(false)
	if warning then
		dialog:SetHeight(220)
		dialog.warning:SetPoint("TOP",dialog.editBox,"BOTTOM",0,-4)
		dialog.warning.text:SetText(warning)
		dialog.warning:Show()
		dialog.editBox:SetText(rematch:GetSidelineContext("sendTo") or "")
	end
	dialog.editBox:SetScript("OnTextChanged",rematch.SendOnTextChanged)
end

-- when the "Who would you like to send this team to?" editbox changes
function rematch:SendOnTextChanged()
	local sendTo = self:GetText()
	rematch:SetSidelineContext("sendTo",sendTo)
	rematch:SetAcceptEnabled(sendTo:len()>0)
end

-- sends the sidelined team to sendTo context
function rematch:SendTeam()
	print("Sending",rematch:GetSidelineTitle(),"to",rematch:GetSidelineContext("sendTo"))
	local dialog = rematch:ShowSendPendingDialog()
	dialog.runOnCancel = rematch.StopSend

	local teamString = rematch:ConvertTeamToString(dialog.teamName)
	if teamString then
		rematch:StartTimer("SendTimeout",5,rematch.SendTimeout)
		rematch:RegisterEvent("CHAT_MSG_SYSTEM")
		-- otherwise send by normal SendAddonMessage
		rematch:SendMessage(teamString,rematch:GetSidelineContext("sendTo"))
	end
end

-- shows either a "Sending..." or "Team received!" (if success true) dialog
function rematch:ShowSendPendingDialog(success)
	local dialog = rematch.dialog
	rematch:ShowDialog("SendTeamPending",128,rematch:GetSidelineTitle(true),success and L["Team received!"] or L["Sending..."],success and true)
	dialog.team:SetPoint("TOP",0,-24)
	dialog.team:Show()
	rematch:FillPetFramesFromTeam(dialog.team.pets,(rematch:GetSideline()))
	if not success then
		dialog.accept:Hide()
	end
	return dialog
end

function rematch:StopSend()
	rematch:UnregisterEvent("CHAT_MSG_SYSTEM")
	rematch:StopTimer("SendTimeout")
end

-- after 5 seconds with no response
function rematch:SendTimeout()
	rematch:SendFailed(L["No response. Lag or no Rematch?"])
end

-- send failed for passed reason, return to the send dialog with the reason
function rematch:SendFailed(reason)
	rematch:StopSend()
	rematch:ShowSendDialog(reason)
end

-- sends message to sendTo via regular SendAddonMessage (bnet support NYI)
function rematch:SendMessage(message,sendTo)
	SendAddonMessage("Rematch",message,"WHISPER",sendTo)
end

-- captures "No player named 'sendTo' is currently playing." system message
function rematch.events.CHAT_MSG_SYSTEM(message)
	if message==format(ERR_CHAT_PLAYER_NOT_FOUND_S,rematch:GetSidelineContext("sendTo")) then
		rematch:SendFailed(L["They do not appear to be online."])
	end
end

-- handles incoming messages via the regular SendAddonMessage
function rematch.events.CHAT_MSG_ADDON(prefix,message,_,sender)
	if prefix=="Rematch" then
		rematch:HandleReceivedMessage(message,sender)
	end
end

-- actually processes the message (bnet support NYI)
function rematch:HandleReceivedMessage(message,sender)
	if message=="ok" then
		rematch:StopSend()
		rematch:ShowSendPendingDialog(true)
	elseif message=="busy" then
		rematch:SendFailed(L["They're busy. Try again later."])
	elseif message=="combat" then
		rematch:SendFailed(L["They're in combat. Try again later."])
	elseif message=="block" then
		rematch:SendFailed(L["They have team sharing disabled."])
	else -- any other messages are unsolicited, likely an incoming team

		local dialog = rematch.dialog
		-- likely this is a team received, first check if user ready to receive a team
		if settings.DisableShare then
			rematch:SendMessage("block",sender)
		elseif InCombatLockdown() then
			rematch:SendMessage("combat",sender)
		elseif dialog:IsVisible() then
			rematch:SendMessage("busy",sender)
		else
			-- user appears ready to receive; now check if it's an actual team
			local key = rematch:ConvertStringToTeam(message)
			if key then
				-- it's an actual team, send back an ok
				rematch:SendMessage("ok",sender)
				if not rematch:IsVisible() then
					rematch:Toggle() -- show rematch if it's not already on screen
				end
				dialog.teamString = message
				rematch:SetSidelineContext("sender",sender)
				rematch:ShowReceiveDialog()
			end
		end

	end
end

-- shows the receive dialog for a sidelined team
function rematch:ShowReceiveDialog()
	local team,key = rematch:GetSideline()
	local dialog = rematch:ShowDialog("ReceiveTeam",170,L["Incoming Rematch Team"],L["Save this team?"],rematch.ReceiveAcceptOnClick)
	dialog.text:SetSize(220,42)
	dialog.text:SetPoint("TOP",0,-18)
	local form
	if type(key)=="number" then
		form = L["\124cffffd200%s\124r has sent you a team for %s."]
	else
		form = L["\124cffffd200%s\124r has sent you a team named \124cffffd200%s\124r."]
	end
	dialog.text:SetText(format(form,rematch:GetSidelineContext("sender"),rematch:GetTeamTitle(key)))
	dialog.text:Show()
	dialog.team:SetPoint("TOP",dialog.text,"BOTTOM",0,-4)
	dialog.team:Show()
	rematch:FillPetFramesFromTeam(dialog.team.pets,team)
	if #settings.TeamGroups>1 then
		dialog.tabPicker:Show()
	end
	-- if team already exists, show confirm frame and make dialog taller to fit
	if rematch:ConfirmKeyExists(key,"TOP",dialog.team,"BOTTOM",0,-8) then
		dialog:SetHeight(240)
	end
end

-- when accept (green check) is clicked on the receive dialog
function rematch.ReceiveAcceptOnClick()
	if rematch.overwriteConfirm.copy:GetChecked() then
		rematch:MakeSidelineUnique()
	end
	local team,teamKey = rematch:GetSideline()
	if saved[teamKey] then
		rematch:ShowReplaceDialog(rematch.ShowReceiveDialog)
	else
		rematch:PushSideline(true)
	end
end
